#--
# Copyright (c) 2008, 2009, 2010 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""Security checks added. When not logged, clicking on the 'Edit this page'
or the 'View the complete list of pages' links raise a security exception
"""

from __future__ import with_statement

import re
import docutils.core

from nagare import component, presentation, var, security, wsgi

from wikidata import PageData

# ---------------------------------------------------------------------------

# A simple `Login` object is defined.
class Login:
    pass

# It only displays the name of the logged user
@presentation.render_for(Login)
def render(self, h, *args):
    user = security.get_user()  # Get the user object
    
    if not user:    # The anonymous user is ``None``
        return h.i('not logged')
    
    return ('Welcome ', h.b(user.id))

# ---------------------------------------------------------------------------

wikiwords = re.compile(r'\b([A-Z]\w+[A-Z]+\w+)')

class Page(object):
    def __init__(self, title):
        self.title = title

    def edit(self, comp):
        content = comp.call(PageEditor(self))

        if content is not None:
            # Our security policy : a user needs to have the ``wiki.editor`` permission
            # to be able to modify the content of a page
            security.check_permissions('wiki.editor', self)            
            page = PageData.get_by(pagename=self.title)
            page.data = content

@presentation.render_for(Page)
def render(self, h, comp, *args):
    page = PageData.get_by(pagename=self.title)

    content = docutils.core.publish_parts(page.data, writer_name='html')['html_body']
    content = wikiwords.sub(r'<wiki>\1</wiki>', content)
    html = h.parse_htmlstring(content, fragment=True)[0]

    for node in html.getiterator():
        if node.tag == 'wiki':
            a = h.a(node.text, href='page/'+node.text).action(lambda title=unicode(node.text): comp.answer(title))
            node.replace(a)
    
    return (html, h.a('Edit this page', href='page/'+self.title).action(lambda: self.edit(comp)))

@presentation.render_for(Page, model='meta')
def render(self, h, comp, *args):
    return ('Viewing ', h.b(self.title), h.br, h.br,
            'You can return to the ', h.a('FrontPage', href='page/FrontPage').action(lambda: comp.answer(u'FrontPage')))

# ---------------------------------------------------------------------------

class PageEditor(object):
    def __init__(self, page):
        self.page = page

@presentation.render_for(PageEditor)
def render(self, h, comp, *args):
    content = var.Var()

    page = PageData.get_by(pagename=self.page.title)

    with h.form:
        with h.textarea(rows='10', cols='40').action(content):
            h << page.data
        h << h.br
        h << h.input(type='submit', value='Save').action(lambda: comp.answer(content()))
        h << ' '
        h << h.input(type='submit', value='Cancel').action(comp.answer)

    return h.root

@presentation.render_for(PageEditor, model='meta')
def render(self, h, *args):
    return ('Editing ', h.b(self.page.title))

# ---------------------------------------------------------------------------

class Wiki(object):
    def __init__(self):
        # The wiki application now has a ``Login`` child
        self.login = component.Component(Login())        
        self.content = component.Component(None)
        self.content.on_answer(self.goto)

        self.goto(u'FrontPage')

    def goto(self, title):
        page = PageData.get_by(pagename=title)
        if page is None:
            PageData(pagename=title, data='')

        self.content.becomes(Page(title))

@presentation.render_for(Wiki)
def render(self, h, comp, *args):
    h.head.css('main_css', '''
    .document:first-letter { font-size:2em }
    .meta { float:right; width: 10em; border: 1px dashed gray;padding: 1em; margin: 1em; }
    .login { font-size:0.75em; }    
    ''')

    # Display the login view
    with h.div(class_='login'):
        h << self.login

    with h.div(class_='meta'):
        h << self.content.render(h, model='meta')
        
    h << self.content << h.hr

    h << 'View the ' << h.a('complete list of pages', href='all').action(lambda: self.goto(comp.call(self, model='all')))
    
    return h.root

# Our security policy : a user needs to have the ``wiki.admin`` permission
# to be able to list all the available pages
@presentation.render_for(Wiki, model='all')
@security.permissions('wiki.admin')
def render(self, h, comp, *args):
    with h.ul:
        for page in PageData.query.order_by(PageData.pagename):
            with h.li:
                h << h.a(page.pagename, href='page/'+page.pagename).action(lambda title=page.pagename: comp.answer(title))

    return h.root

# ---------------------------------------------------------------------------

@presentation.init_for(Wiki, "(len(url) == 2) and (url[0] == 'page')")
def init(self, url, *args):
    title = url[1]
    
    page = PageData.get_by(pagename=title)
    if page is None:
        raise presentation.HTTPNotFound()

    self.goto(title)
    
@presentation.init_for(Wiki, "len(url) and (url[0] == 'all')")
def init(self, url, comp, *args):
    component.call_wrapper(lambda: self.goto(comp.call(self, model='all')))

# ---------------------------------------------------------------------------

app = Wiki



#--
# Copyright (c) 2008, 2009, 2010 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""Example to demonstrate the use of asynchronous rendering of
components in a portal-like container
"""

from __future__ import with_statement

from nagare import component, presentation, ajax
from nagare.namespaces import xhtml

from nagare.examples import counter, calculator, tictactoe
from nagare.examples.wiki import wiki9
from nagare.examples.gallery import gallery7
from nagare.examples.jewels import Jewels

class Portal(object):
    """The ``Portal`` object is only a container"""

    def __init__(self):
        """Each components to display is simply an attribute"""
        self.calc = component.Component(calculator.Calculator())
        self.counter = component.Component(counter.Counter1())
        self.tictactoe = component.Component(tictactoe.Task())
        self.wiki = component.Component(wiki9.app())
        self.jewels = component.Component(Jewels(15, 10))
        self.gallery = component.Component(gallery7.app())


@presentation.render_for(Portal)
def render(self, h, *args):
    """The portal view uses YUI to display a portal-like structure with
    drag and drop aware portlets
    """
    h.head.javascript_url(ajax.YUI_EXTERNAL_PREFIX + '/yahoo-dom-event/yahoo-dom-event.js')
    h.head.javascript_url(ajax.YUI_EXTERNAL_PREFIX + '/dragdrop/dragdrop.js')
    h.head.javascript_url(ajax.YUI_EXTERNAL_PREFIX + '/dom/dom.js')

    h.head.javascript_url('portal.js')
    h.head.css_url('portal.css')

    h.head << h.head.title('Portal demonstration')

    with h.div(id='portal', class_='FixedWidth'):

        # Each column must be a <div> with the class 'portletColumn'
        with h.div(class_='portletColumn'):
            # Each portlet must be a <div> with the class 'portlet'
            with h.div(class_='portlet'):
                # Each D&D portlet handle must be a <div> with the class 'portletHandle'
                h << h.div('About', class_='portletHandle')
                with h.div:
                    h << h.p('This is a simple prototype of a dashboard that allows you to reorder containers much like iGoogle')
                    h << h.p('Select a container and drag it to another column.  As you do so, a marker appears between the place where to drop the element.')
                    h << h.p('Drag and drop is facilitated by the ', h.a('YUI Library.', href='http://developer.yahoo.com/yui/'))

            with h.div(class_='portlet'):
                h << h.div('Counter', class_='portletHandle')
                # Render the default asynchronous view of the component
                h << self.counter.render(xhtml.AsyncRenderer(h))

            with h.div(class_='portlet'):
                h << h.div('RPN Calculator', class_='portletHandle')
                h << self.calc.render(xhtml.AsyncRenderer(h))

        with h.div(class_='portletColumn'):
            with h.div(class_='portlet'):
                h << h.div('TicTacToe', class_='portletHandle')
                h << self.tictactoe.render(xhtml.AsyncRenderer(h))

            with h.div(class_='portlet'):
                h << h.div('Wiki', class_='portletHandle')
                h << self.wiki.render(xhtml.AsyncRenderer(h))
                h << h.p(style='clear: both')

            with h.div(class_='portlet'):
                h << h.div('Jewels', class_='portletHandle')

                h.head.css_url('jewels.css')
                h << h.div(self.jewels.render(xhtml.AsyncRenderer(h)), class_='jewels')

                h << h.p(style='clear: both')

        with h.div(class_='portletColumn'):
            with h.div(class_='portlet'):
                h << h.div('Photos gallery', class_='portletHandle')
                h << self.gallery.render(xhtml.AsyncRenderer(h))
                h << h.p(style='clear: both')

        # Initialize the portal
        h << h.script('portalInit("portal")', type='text/javascript')

    return h.root

# ---------------------------------------------------------------------------

app = Portal



#--
# Copyright (c) 2008, 2009, 2010 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from __future__ import with_statement

from webob import exc
from nagare import component, presentation

# Modules where the examples are located
import counter
import calculator
import color
import tictactoe
import form
import number

# ---------------------------------------------------------------------------

class TreeView(object):
    """This component display a 2-levels tree
    """
    def __init__(self, nodes, label):
        """Initialization
        
        In:
          - ``nodes`` -- list of list
          - ``label`` -- prefix label to display for the 2nd level nodes
        """
        self.nodes = nodes
        self.label = label
        
        self._selected = 0   # The currently selected nodes
        
    def selected(self, comp, i, n, name, data):
        """A new 2nd level node is selected
        
        In:
          - ``comp`` -- this component
          - ``i`` -- index of the selected node
          - ``n`` -- the new selected node
          - ``name`` -- label of the first level node (the parent)
          - ``data`` -- data of the selected 2nd level node
        """
        self._selected = n
        comp.answer((name, i, data))
        
    def select(self, comp, category, n):
        node = 0
        for (name, level2_nodes) in self.nodes:
            if name == category:
                if n < len(level2_nodes):
                    node += n
                    self.selected(comp, n, node, name, level2_nodes[n])
                    return True
            else:
                node += len(level2_nodes)

        return False
        
        
@presentation.render_for(TreeView)
def render(self, h, comp, *args):
    node = 0

    # Display a list of lists
    with h.ul(style='padding-right: 10px; border-right: solid 3px #eee'):
        for (name, level2_nodes) in self.nodes:
            # First level, display the name of the node
            with h.li(name.capitalize()):
                with h.ul:
                    for (i, data) in enumerate(level2_nodes):
                        # Second level, display a selectable label
                        with h.li as li:
                            if node == self._selected:
                                # Currently selected node
                                li.set('style', 'background-color: #eee')
                                h << self.label + str(i+1)
                            else:
                                # Selectable node
                                h << h.a(self.label + str(i+1), href='%s/%d' % (name, i+1)).action(lambda i=i, node=node, name=name, data=data: self.selected(comp, i, node, name, data))

                            node += 1

    return h.root

# ---------------------------------------------------------------------------

class Examples(object):
    """Display the UI to browse the examples
    """
    def __init__(self, title, label, repository_path, examples):
        """Initialization
        
        In:
          - ``title`` -- title to display
          - ``label`` -- prefix label to display for the 2nd level nodes 
          - ``repository_path`` -- url template to display the module code
          - ``examples`` -- tree of examples
        """
        self.title = title
        self.repository_path = repository_path
        
        # The ``example`` component, displayed on the right side of the
        # screen. It will be replaced each time an example is selected
        self.example = component.Component(None)

        # Display the first example found
        self.change_example((examples[0][0], 0, examples[0][1][0]))
        
        # The ``tree_view`` component, displayed on the left side of the
        # screen. It will answered the selected example
        self.tree_view = component.Component(TreeView(examples, label))
        
        # Each time an example is selected into the tree view, call the
        # ``change_example()`` method
        self.tree_view.on_answer(self.change_example)

    def change_example(self, (module_name, i, (description, comp))):
        """A new example is selected
        
        In:
          - ``module_name`` -- name of the module where the example is located
          - ``description`` -- description of the example
          - ``comp`` -- root component of the example
        """
        self.module_name = module_name
        self.description = description

        # Change the right side of the screen with the example
        self.example.becomes(comp(), url='%s/%d' % (module_name, i+1))
        
@presentation.render_for(Examples)
def render(self, h, comp, *args):
    """Display the full UI
    """
    # Add some CSS into the page header
    h.head.css_url('/static/nagare/application.css')    
    h.head.css('examples', '''
    #description {
        border: dashed 1px;
        padding: 5px;
        background-color: #eee;
        font-style: italic;
        width: 95%;
    }
    
    #example {
        padding-left: 10px;
        padding-top: 25px;
        vertical-align: top;
        width: 100%;
    }
    
    #source {
        text-align: right;
        font-size: 0.7em;
        padding-right: 30px;
    }
    ''')
    h.head << h.head.title('Nagare examples')

    with h.div(class_='mybody'):
        # Create the page banner
        with h.div(id='myheader'):
            h << h.a(h.img(src='/static/nagare/img/logo.gif'), id='logo', href='http://www.nagare.org/', title='Nagare home')
            h << h.span(self.title, id='title')
    
        with h.div(id='main'):
            # Create a table with, into the left column, the tree view of the examples
            # and, into the right column the currently selected example
            with h.table(width='100%'):
                with h.tr:
                    # Display the tree view
                    h << h.td(self.tree_view, valign='top')
                    
                    # Display the currently selected example
                    with h.td(id='example'):
                        h << h.div(self.description, id='description')
                        with h.div(id='source'):
                            h << h.a('View the source', href=self.repository_path % self.module_name)
                            h << h.br
                        h << self.example

    h << h.div(class_='footer')

    return h.root

# ---------------------------------------------------------------------

@presentation.init_for(Examples, "http_method != 'GET'")
def init(self, url, comp, http_method, request):
    raise exc.HTTPMethodNotAllowed()

@presentation.init_for(Examples, "len(url) >= 2 and http_method == 'GET'")
def init(self, url, comp, http_method, request):
    (name, n) = url[:2]
    r = self.tree_view().select(self.tree_view, name, int(n)-1)
    if not r:
        raise exc.HTTPNotFound()
    
    if len(url) > 2:
        self.example.init(url[2:], http_method, request)
    
# ---------------------------------------------------------------------

def module_name(name):
    """From a full Python module name, return only the module name
    
    In:
      - ``name`` -- Python module name
      
    Return:
      - module name
    """
    # Keep only the last part of the path
    return name.rsplit('.', 1)[1]


# Modules where to located the ``examples`` list
modules = (counter, calculator, color, tictactoe, number, form)

# Create a tree with, at the first level, the module name and, at the second
# level, a list of tuples (example description, example factory)
examples = [(module_name(module.__name__), zip(module.examples[::2], module.examples[1::2])) for module in modules]

# TRAC url to display a module code from the SVN repository
SVN_EXAMPLES_URL = 'http://www.nagare.org/trac/browser/trunk/nagare/examples/nagare/examples/%s.py'

app = lambda: Examples('Demo', 'Demo', SVN_EXAMPLES_URL, examples)



#--
# Copyright (c) 2008, 2009, 2010 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import pkg_resources

pkg_resources.declare_namespace('nagare')

